import java.awt.Dimension;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.io.File;
import java.io.FilenameFilter;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;


public class GeneralConfig {
	
	private Value title;
	private Value sourceFolder;
	private Value otherFolder;
	private int itemCols;
	
	private List<Sensor> sensors;
	private FormUtils fu;
	private ConfigFile configFile;
	
	private ActivityMonitor monitor;
	private LogWindow logWindow;
	
	private int dayNr;
	
	private static final String TITLE = "Title:";
	private static final String SOURCE_FOLDER = "Source folder:";
	private static final String OTHER_FILES_FOLDER = "Unmatched files folder:";
	private static final String NR_COLUMNS = "Columns:";

	public GeneralConfig(ConfigFile configFile) {
		Calendar calendar = new GregorianCalendar();
		this.dayNr = calendar.get(Calendar.DAY_OF_MONTH); // avoid deleting on startup
		this.configFile = configFile;
		
		title = new Value("");
		sourceFolder = new Value("");
		otherFolder = new Value("");
		
		sensors = new ArrayList<Sensor>();
		fu = new FormUtils(this);
		
		if (configFile.nrLines() > 0)
			read();
		else {
			configFile.newFile();
			configFile.writeToken("Group Name:", "Sample Group", null);
			configFile.writeToken("Item Name:", "Sample Item", null);
			configFile.rewind();
			read();
			store();
		}
	}

	public void read() {
		this.title.setVal(configFile.readToken(TITLE, "Data Manager"));
		this.sourceFolder.setVal(configFile.readToken(SOURCE_FOLDER, "C:\\"));
		this.otherFolder.setVal(configFile.readToken(OTHER_FILES_FOLDER, ""));
		this.itemCols = Integer.parseInt(configFile.readToken(NR_COLUMNS, "2"));

		Sensor sensor = new Sensor();
		while (sensor.read(configFile)) {
			sensors.add(sensor);
			sensor = new Sensor();
		}		
	}
	
	public void write() {
		fu.updateData(true);
		configFile.writeToken("This file will be automatically overwritten the next time you run the program!", "", "");
		configFile.writeToken("You can make edits to this file when the program is not running", "", "");
		configFile.writeToken("but it is no use to change the layout of this file or add your own comments.", "", "");
		configFile.writeToken("", "", null);
		configFile.writeToken("Uncomment lines to let them take effect", "", "");
		configFile.writeToken("", "", null);
		configFile.writeToken(TITLE, title.getStringVal(), "Data Manager");
		configFile.writeToken(SOURCE_FOLDER, sourceFolder.getStringVal(), "C:\\");
		configFile.writeToken(OTHER_FILES_FOLDER, otherFolder.getStringVal(), "");
		configFile.writeToken(NR_COLUMNS, "" + itemCols, "2");
		configFile.writeln();
		for (Sensor sensor : sensors)
			sensor.write(configFile);
	}
	
	public String getTitle() {
		return title.getStringVal();
	}

	public void addToPanel(final Frame parent) {
		JPanel formPanel = fu.createGridPanel(null);
		
		// general input
		JPanel generalInput = fu.createGridPanel("Input folders");
		int ypos = 0;
		fu.addBrowseFolder(generalInput, SOURCE_FOLDER, sourceFolder, 0, ypos++);
		fu.addBrowseFolder(generalInput, OTHER_FILES_FOLDER, otherFolder, 0, ypos++);
		fu.addPanel(formPanel, generalInput, 0, 0);
//		formPanel.add(generalInput);

		// control
		ypos = 0;
		JPanel controlPanel = fu.createGridPanel("Activity");
		final JButton actionButton = fu.addActionButton(controlPanel, "Start", 1, ypos);
		final JButton showLogWindowButton = fu.addActionButton(controlPanel, "View Log", 2, ypos);
		JTextField activityText = fu.addTextBox(controlPanel, "", "", new Value(""), false, 0, ypos++);
		activityText.setMinimumSize(new Dimension(200,20));
		activityText.setPreferredSize(new Dimension(200,20));
//		JTextField messageText = fu.addTextBox(controlPanel, "", "", new Value(""), false, 2, ypos++);
		JProgressBar progressBar = fu.addProgressBar(controlPanel, 0, ypos);
		progressBar.setMinimumSize(new Dimension(200,20));
		progressBar.setPreferredSize(new Dimension(200,20));
		fu.addPanel(formPanel, controlPanel, 1, 0);
//		formPanel.add(controlPanel);
		monitor = new ActivityMonitor(progressBar, activityText);
		logWindow = new LogWindow(parent);

		// tabs
		JTabbedPane tabs = new JTabbedPane();
		for (Sensor sensor : sensors) {
			JPanel sensorPanel = new JPanel();
			sensor.addToPanel(sensorPanel, fu, itemCols);
			tabs.addTab(sensor.getName(), sensorPanel);
		}
		fu.addPanelEntireRow(formPanel, tabs, 0, 1);
//		formPanel.add(tabs);
		
		parent.add(formPanel);
		
		final GeneralConfig generalConfig = this;
		actionButton.addActionListener(new ActionListener() {
			FileProcessor processor = null;
			public void actionPerformed(ActionEvent event) {
				if (event.getSource() instanceof JButton) {
					if (processor == null) {
						fu.enableInput(false);
						fu.updateData(true);
						String msg = checkData();
						if (msg.length() > 0) {
							messageBox(parent, "Incorrect input", msg);
							fu.enableInput(true);
						}
						else {
							actionButton.setText("Abort");
							logWindow.log("Started", false);
							processor = new FileProcessor(generalConfig, logWindow);
							processor.addActionListener(this);
						}
					} else {
						processor.abort();
						actionButton.setText("Aborting...");
						actionButton.setEnabled(false);
					}
				} else if (event.getSource() instanceof FileProcessor) {
					String msg = event.getActionCommand();
					if (msg != null && msg.length() > 0 && msg.compareTo("Ready") != 0) {
						logWindow.log("Error: " + msg, true);
						messageBox(parent, "Error", msg);
					}
					processor = null;
					logWindow.log("Stopped", false);
					actionButton.setText("Start");
					actionButton.setEnabled(true);
					fu.enableInput(true);
				}
			}
		});
		showLogWindowButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				logWindow.ensureLogWindowShowing();
			}
		});
		parent.addComponentListener(new ComponentListener() {
			public void componentHidden(ComponentEvent event) {
			}

			public void componentMoved(ComponentEvent event) {
				logWindow.parentMoved();
			}

			public void componentResized(ComponentEvent event) {
			}

			public void componentShown(ComponentEvent event) {
			}
		});
	}
	
	public void processFiles() {
		// Init
		for (Sensor sensor : sensors)
			sensor.initFileProcessing();
		
		// Filter
		monitor.startJob("Reading source folder");
		File source = new File(sourceFolder.getStringVal());
		FilenameFilter filter = new FilenameFilter() {
			public boolean accept(File parent, String fileName) {
				File file = new File(parent, fileName);
				if (file.isDirectory())
					return false;

				for (Sensor sensor : sensors)
					if (sensor.acceptFile(file))
						return false;
				return true;
			}
			
		};
		File [] unprocessedFiles = source.listFiles(filter);
		monitor.endJob();
		
		// Process
		for (Sensor sensor : sensors)
			sensor.processFiles(monitor, logWindow);
		
		// Move "unmatched" files
		if (unprocessedFiles != null) {
			monitor.startJob("Moving unmatched files", unprocessedFiles.length);
			if (otherFolder.getStringVal() != null && otherFolder.getStringVal().length() > 0) {
				for (File file : unprocessedFiles) {
					File destinationFile = new File(new File(otherFolder.getStringVal()), file.getName());
					if (destinationFile.exists())
						destinationFile.delete();
					boolean success = file.renameTo(destinationFile);
					if (!success)
						logWindow.log("Could not move " + file.getAbsolutePath() + " to " + otherFolder.getStringVal(), true);
					monitor.work();
				}
			}
			monitor.endJob();
		} else
			logWindow.log("Could not move unmatched files", true);

		// Delete old files at midnight
		Calendar calendar = new GregorianCalendar();
		int day = calendar.get(Calendar.DAY_OF_MONTH);
		if (day != dayNr) {
			dayNr = day; 
			for (Sensor sensor : sensors)
				sensor.deleteOldFiles(monitor, logWindow);
		}
	}

	public void store() {
		configFile.newFile();
		write();
		configFile.store();
	}
	
	private String checkData() {
		if (!new File(sourceFolder.getStringVal()).exists())
			return "Please ensure that the source folder exists.";

		if (!new File(otherFolder.getStringVal()).exists())
			return "Please ensure that the folder for unmatched files exists.";

		for (Sensor sensor : sensors) {
			String msg = sensor.checkData();
			if (msg.length() > 0)
				return msg;			
		}
		
		// All OK
		return "";
	}
	
	private void messageBox(Frame parent, String title, String msg) {
		JOptionPane.showMessageDialog(parent, msg, title, JOptionPane.WARNING_MESSAGE);
	}
	
//	private void messageBox1(Frame parent, String title, String msg) {
//		final JDialog dlg = new JDialog(parent, title, true);
//		JPanel dlgPanel = new JPanel();
//		JLabel label = new JLabel(msg);
//		label.setBorder(BorderFactory.createEmptyBorder(20, 20, 20, 20));
//		dlgPanel.add(label);
//		JButton okButton = new JButton("OK");
//		okButton.addActionListener(new ActionListener() {
//			public void actionPerformed(ActionEvent e) {
//				dlg.setVisible(false);
//			}
//		});
//		dlgPanel.add(okButton);
//		dlg.add(dlgPanel);
//		dlg.pack();
//		int xpos = parent.getX();
//		int ypos = parent.getY();
//		int xsize = parent.getWidth();
//		int ysize = parent.getHeight();
//		xsize -= dlg.getWidth();
//		ysize -= dlg.getHeight();
//		dlg.setLocation(xpos + xsize / 2, ypos + ysize / 2);
//		dlg.setVisible(true); // this blocks until JDialog closes
//	}
}
